import json
from datetime import date, datetime
from typing import Final
from .env_pomes import APP_PREFIX, env_get_str

VALIDATION_MSG_LANGUAGE: Final[str] = env_get_str(f"{APP_PREFIX}_VALIDATION_MSG_LANGUAGE", "pt")
VALIDATION_MSG_PREFIX: Final[str] = env_get_str(f"{APP_PREFIX}_VALIDATION_MSG_PREFIX", APP_PREFIX)


# valida o valor do atributo de acordo com lista e restrições de comprimento
def validate_value(value: str | int | float, min_val: int = None,
                   max_val: int = None, default: bool | list[any] = None) -> str:

    # inicializa variável de retorno
    result: str | None = None
    # 'value' pode ser None, e None pode estar em 'default'
    if isinstance(default, list):
        if value not in default:
            if value is None:
                result = __format_error(10)
            else:
                length: int = len(default)
                if length == 1:
                    result = __format_error(15, value, default[0])
                else:
                    # o último elemento da lista é None ?
                    if default[-1] is None:
                        # sim, omita-o da mensagem
                        length -= 1
                    result = __format_error(16, value, [default[:length]])
    elif value is None:
        if isinstance(default, bool) and default:
            result = __format_error(10)
    elif isinstance(value, str):
        length: int = len(value)
        if min_val is not None and max_val == min_val and length != min_val:
            result = __format_error(14, value, min_val)
        elif max_val is not None and max_val < length:
            result = __format_error(13, value, max_val)
        elif min_val is not None and length < min_val:
            result = __format_error(12, value, min_val)
    elif (min_val is not None and value < min_val) or \
         (max_val is not None and value > max_val):
        result = __format_error(17, value, [min_val, max_val])

    return result


# obtem e valida booleano - valor retornado pode ser None
def validate_bool(errors: list[str], scheme: dict, attr: str,
                  default: bool = None, mandatory: bool = False, parse_string: bool = False) -> bool:

    result: bool | None = None
    stat: str | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]
    try:
        if parse_string:
            result = json.loads(scheme[suffix].lower())
        else:
            result = scheme[suffix]

        if not isinstance(result, bool):
            stat = __format_error(18, result, "bool")
    except (KeyError, TypeError):
        if default is not None:
            result = default
        elif mandatory:
            stat = __format_error(10)

    if stat is not None:
        errors.append(f"{stat} @{attr}")

    return result


# obtem e valida inteiro - valor retornado pode ser None
def validate_int(errors: list[str], scheme: dict, attr: str,
                 min_value: int = None, max_value: int = None,
                 default: bool | int | list[int] = None) -> int:

    stat: str | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]

    # obtem e valida o valor
    result: int | None = scheme.get(suffix)
    if result is not None and \
       (not isinstance(result, int) or isinstance(result, bool)):
        stat = __format_error(18, result, "int")
    # bool é subtipo de int
    elif isinstance(default, int) and not isinstance(default, bool):
        if result is None:
            result = default
        else:
            stat = validate_value(result, min_value, max_value)
    else:
        stat = validate_value(result, min_value, max_value, default)

    if stat is not None:
        errors.append(f"{stat} @{attr}")

    return result


# obtem e valida decimal - valor retornado pode ser None
def validate_float(errors: list[str], scheme: dict, attr: str,
                   min_value: float = None, max_value: float = None,
                   default: bool | float | list[float | int] = None) -> float:

    stat: str | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]

    # obtem e valida o valor
    result: float | None = scheme.get(suffix)
    if result is not None and not isinstance(result, int) and not isinstance(result, float):
        stat = __format_error(18, result, "float")
    else:
        # bool é subtipo de int
        if isinstance(result, int) and not isinstance(result, bool):
            result = float(result)
        if isinstance(default, float):
            if result is None:
                result = default
            else:
                stat = validate_value(result, min_value, max_value)
        else:
            stat = validate_value(result, min_value, max_value, default)

    if stat is not None:
        errors.append(f"{stat} @{attr}")

    return result


# obtem e valida string - valor retornado pode ser None
def validate_str(errors: list[str], scheme: dict, attr: str,
                 min_length: int = None, max_length: int = None,
                 default: bool | str | list[str] = None) -> str:

    stat: str | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]

    # obtem e valida o valor
    result: str = scheme.get(suffix)
    if result is not None and not isinstance(result, str):
        stat = __format_error(18, result, "str")
    elif isinstance(default, str):
        if result is None:
            result = default
        else:
            stat = validate_value(result, min_length, max_length)
    else:
        stat = validate_value(result, min_length, max_length, default)

    if stat is not None:
        errors.append(f"{stat} @{attr}")

    return result


# obtem e valida data - valor retornado pode ser None
def validate_date(errors: list[str], scheme: dict, attr: str,
                  default: bool | date = None, day_first: bool = True) -> date:

    # import needed module
    from .datetime_pomes import date_parse

    result: date | None = None
    stat: str | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]
    try:
        date_str: str = scheme[suffix]
        result = date_parse(date_str, dayfirst=day_first)
        if result is None:
            stat = __format_error(11, date_str)
        elif result > datetime.now().date():
            stat = __format_error(19, date_str)
    except KeyError:
        if isinstance(default, bool) and default:
            stat = __format_error(10)
        elif isinstance(default, date):
            result = default

    if stat is not None:
        errors.append(f"{stat} @{attr}")

    return result


# obtem e valida data - valor retornado pode ser None
def validate_datetime(errors: list[str], scheme: dict, attr: str,
                      default: bool | datetime = None, day_first: bool = True) -> datetime:

    # import needed module
    from .datetime_pomes import datetime_parse

    result: datetime | None = None
    stat: str | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]
    try:
        date_str: str = scheme[suffix]
        result = datetime_parse(date_str, dayfirst=day_first)
        if result is None:
            stat = __format_error(21, date_str)
        elif result > datetime.now():
            stat = __format_error(18, date_str)
    except KeyError:
        if isinstance(default, bool) and default:
            stat = __format_error(10)
        elif isinstance(default, datetime):
            result = default

    if stat is not None:
        errors.append(f"{stat} @{attr}")

    return result


# obtem e valida lista de inteiros - valor retornado pode ser None ou lista vazia
def validate_ints(errors: list[str], scheme: dict, attr: str,
                  min_size: int = None, max_size: int = None, mandatory: bool = False) -> list[int]:

    result: list[any] | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]
    try:
        values: list[any] = scheme[suffix]
        if isinstance(values, list):
            result = []
            if len(values) == 0:
                raise KeyError
            else:
                for inx, value in enumerate(values):
                    result.append(value)
                    if isinstance(value, int):
                        stat: str = validate_value(value, min_size, max_size)
                    else:
                        stat: str = __format_error(18, value, "int")
                    if stat is not None:
                        errors.append(f"{stat} @{attr}[{inx+1}]")
        else:
            errors.append(__format_error(18, result, "list", f"@{attr}"))
    except (KeyError, TypeError):
        if mandatory:
            errors.append(__format_error(10, f"@{attr}"))

    return result


# obtem e valida lista de strings - valor retornado pode ser None ou uma lista vazia
def validate_strs(errors: list[str], scheme: dict, attr: str,
                  min_size: int, max_size: int, mandatory: bool = False) -> list[str]:

    result: list[any] | None = None
    pos: int = attr.rfind(".") + 1
    suffix: str = attr[pos:]
    try:
        values: list[any] = scheme[suffix]
        if isinstance(values, list):
            result = []
            if len(values) == 0:
                raise KeyError
            else:
                for inx, value in enumerate(values):
                    result.append(value)
                    if isinstance(value, str):
                        stat: str = validate_value(value, min_size, max_size)
                    else:
                        stat: str = __format_error(18, value, "str")
                    if stat is not None:
                        errors.append(f"{stat} @{attr}[{inx+1}]")
        else:
            errors.append(__format_error(18, result, "list", f"@{attr}"))
    except (KeyError, TypeError):
        if mandatory:
            errors.append(__format_error(11, f"@{attr}"))

    return result


def validate_format_error(error_id: int, err_msgs: dict, *args) -> str:

    # inicializa a variável de retorno
    result: str = VALIDATION_MSG_PREFIX + str(error_id) + ": " + err_msgs.get(error_id)

    if result is not None:
        # aplica os argumentos fornecidos
        for arg in args:
            if arg is None:
                result = result.replace(" {}", "", 1)
            elif isinstance(arg, str) and arg.startswith("@"):
                result += " " + arg
            elif isinstance(arg, str) and arg.find(" ") > 0:
                result = result.replace("{}", arg, 1)
            else:
                result = result.replace("{}", f"'{str(arg)}'", 1)

    return result


# formata itens na lista de erros: <codigo> <descricao> [@<atributo>]
def validate_format_errors(errors: list[str]) -> list[dict]:

    # import needed modulo
    from .str_pomes import str_find_whitespace

    # inicializa a variável de retorno
    result: list[dict] = []

    # extrai código, descrição e atributo do texto
    for error in errors:
        # localiza o último indicador do atributo
        pos = error.rfind("@")

        # existe whitespace no nome do atributo ?
        if pos > 0 and str_find_whitespace(error[pos:]) > 0:
            # sim, desconsidere-o
            pos = -1

        # o texto contem o atributo ?
        if pos == -1:
            # não
            out_error: dict = {}
            desc: str = error
        else:
            # sim
            term: str = "attribute" if VALIDATION_MSG_LANGUAGE == "en" else "atributo"
            out_error: dict = {term: error[pos + 1:]}
            desc = error[:pos - 1]

        # o texto contem o código ?
        if desc.startswith(VALIDATION_MSG_PREFIX):
            # sim
            term: str = "code" if VALIDATION_MSG_LANGUAGE == "en" else "codigo"
            out_error[term] = desc[0:7]
            desc = desc[9:]

        term: str = "description" if VALIDATION_MSG_LANGUAGE == "en" else "descricao"
        out_error[term] = desc
        result.append(out_error)

    return result


def __format_error(err_id: int, *args) -> str:

    err_msgs_en: Final[dict] = {
        10: "Value must be provided",
        11: "Invalid value {}",
        12: "Invalid value {}: length shorter than {}",
        13: "Invalid value {}: length longer than {}",
        14: "Invalid value {}: length must be {}",
        15: "Invalid value {}: must be {}",
        16: "Invalid value {}: must be one of {}",
        17: "Invalid value {}: must be in the range {}",
        18: "Invalid value {}: must be type {}",
        19: "Invalid value {}: date is later than the current date"
    }

    err_msgs_pt: Final[dict] = {
        10: "Valor deve ser fornecido",
        11: "Valor {} inválido",
        12: "Valor {} inválido: comprimento menor que {}",
        13: "Valor {} inválido: comprimento maior que {}",
        14: "Valor {} inválido: comprimento deve ser {}",
        15: "Valor {} inválido: deve ser {}",
        16: "Valor {} inválido: deve ser um de {}",
        17: "Valor {} inválido: deve estar no intervalo {}",
        18: "Valor {} inválido: deve ser do tipo {}",
        19: "Valor {} inválido: data posterior à data atual"
    }

    err_msgs: dict | None = None
    match VALIDATION_MSG_LANGUAGE:
        case "en":
            err_msgs = err_msgs_en
        case "pt":
            err_msgs = err_msgs_pt

    return validate_format_error(err_id, err_msgs, args)



from typing import Final

# https://developer.mozilla.org/pt-BR/docs/Web/HTTP/Status

MIMETYPE_BINARY: Final[str] = "application/octet-stream"
MIMETYPE_CSS: Final[str] = "text/css"
MIMETYPE_CSV: Final[str] = "text/csv"
MIMETYPE_HTML: Final[str] = "text/html"
MIMETYPE_JAVASCRIPT: Final[str] = "text/javascript"
MIMETYPE_JSON: Final[str] = "application/json"
MIMETYPE_MULTIPART: Final[str] = "multipart/form-data"
MIMETYPE_PDF: Final[str] = "application/pdf"
MIMETYPE_PKCS7: Final[str] = "application/pkcs7-signature"
MIMETYPE_SOAP: Final[str] = "application/soap+xml"
MIMETYPE_TEXT: Final[str] = "text/plain"
MIMETYPE_URLENCODED: Final[str] = "application/x-www-form-urlencoded"
MIMETYPE_XML: Final[str] = "application/xml"
MIMETYPE_ZIP: Final[str] = "application/zip"


# TODO (adddescription)
__HTTP_STATUS: Final[dict] = {
  200: {
    "name": "OK",
    "description": ""
  },
  201: {
    "name": "CREATED",
    "description": ""
  },
  202: {
    "name": "ACCEPTED",
    "description": ""
  },
  203: {
    "name": "NON AUTHORITATIVE INFORMATION",
    "description": ""
  },
  204: {
    "name": "NO CONTENT",
    "description": ""
  },
  205: {
    "name": "RESET CONTENT",
    "description": ""
  },
  206: {
    "name": "PARTIAL CONTENT",
    "description": ""
  },
  300: {
    "name": "MULTIPLE CHOICE",
    "description": ""
  },
  301: {
    "name": "MOVED PERMANENTLY",
    "description": ""
  },
  302: {
    "name": "FOUND",
    "description": ""
  },
  303: {
    "name": "SEE OTHER",
    "description": ""
  },
  304: {
    "name": "NOT MODIFIED",
    "description": ""
  },
  305: {
    "name": "USE PROXY",
    "description": ""
  },
  307: {
    "name": "TEMPORARY REDIRECT",
    "description": ""
  },
  308: {
    "name": "PERMANENT REDIRECT",
    "description": ""
  },
  400: {
    "name": "BAD REQUEST",
    "description": ""
  },
  401: {
    "name": "UNAUTHORIZED",
    "description": ""
  },
  403: {
    "name": "FORBIDDEN",
    "description": ""
  },
  404: {
    "name": "NOT FOUND",
    "description": ""
  },
  405: {
    "name": "METHOD NOT ALLOWED",
    "description": ""
  },
  406: {
    "name": "NOT ACCEPTABLE",
    "description": ""
  },
  407: {
    "name": "AUTHENTICATION REQUIRED",
    "description": ""
  },
  408: {
    "name": "",
    "description": ""
  },
  409: {
    "name": "REQUEST TIMEOUT",
    "description": ""
  },
  410: {
    "name": "GONE",
    "description": ""
  },
  411: {
    "name": "LENGTH REQUIRED",
    "description": ""
  },
  412: {
    "name": "",
    "description": ""
  },
  413: {
    "name": "PAYLOAD TOO LARGE",
    "description": ""
  },
  414: {
    "name": "URI TOO LONG",
    "description": ""
  },
  500: {
    "name": "INTERNAL SERVER ERROR",
    "description": ""
  },
  501: {
    "name": "NOT IMPLEMENTED",
    "description": ""
  },
  502: {
    "name": "",
    "description": ""
  },
  503: {
    "name": "BAD GATEWAY",
    "description": ""
  },
  504: {
    "name": "GATEWAY TIMEOPUT",
    "description": ""
  },
  505: {
    "name": "",
    "description": ""
  },
  506: {
    "name": "VARIANT ALSO NEGOTIATES",
    "description": ""
  },
  507: {
    "name": "INSUFFICIENT STORAGE",
    "description": ""
  },
  508: {
    "name": "LOOP DETECTED",
    "description": ""
  },
  510: {
    "name": "NOT EXTENDED",
    "description": ""
  },
  511: {
    "name": "NETWORK AUTHENTICATION REQUIRED",
    "description": ""
  }
}


def http_status(status_code: int) -> str:

    item: dict = __HTTP_STATUS.get(status_code, {"name": None})
    result = f"HTTP status code {status_code}, {item.get('name')}"
                                                             
    return result



from datetime import date, datetime
from dateutil import parser
from typing import Final

# some useful date/datetime formats
DATE_FORMAT_STD: Final[str] = "%m/%d/%Y"
DATE_FORMAT_COMPACT: Final[str] = "%Y%m%d"
DATE_FORMAT_INV: Final[str] = "%Y-%m-%d"
DATETIME_FORMAT_STD: Final[str] = "%m/%d/%Y %H:%M:%S"
DATETIME_FORMAT_COMPACT: Final[str] = "%Y%m%d%H%M%S"
DATETIME_FORMAT_INV: Final[str] = "%Y-%m-%d %H:%M:%S"


def date_reformat(dt_str: str, to_format: str, **kwargs: any) -> str:
    """
    Convert the date in *dt_str* to the format especified in *to_format*.
    The argument *dt_str* must represent a valid date, with or without time-of-day information.
    The argument *to_format* must be a valid format for *date* ou *datetime*.

    :param dt_str: The date to convert.
    :param to_format: The format for the conversion.
    :param kwargs: Optional arguments for the parser in python-dateutil ('dayfirst' is a common argument).
    :return:  The converted date.
    """
    result: str | None = None
    ts: datetime = parser.parse(dt_str, **kwargs)
    if ts is not None:
        result = datetime.strftime(ts, to_format)

    return result


def date_parse(dt_str: str, **kwargs: any) -> date:
    """
    Obtain and return the *date* object corresponding to *dt_str*.
    Return *None* se *dt_str* does not contain a valid date.

    :param dt_str: The date, in a supported format.
    :param kwargs: Optional arguments for the parser in python-dateutil ('dayfirst' is a common argument).
    :return: The corresponding date object, or None.
    """
    result: date | None = None
    if dt_str is not None:
        result = parser.parse(dt_str, **kwargs).date()

    return result


def datetime_parse(dt_str: str, **kwargs: any) -> datetime:
    """
   Obtain and return the *datetime* object corresponding to *dt_str*.
   Return *None* se *dt_str* does not contain a valid date.

    :param dt_str: The date, in a supported format.
    :param kwargs: Optional arguments for the parser in python-dateutil ('dayfirst' is a common argument).
    :return: The corresponding datetime object, or None.
    """
    result: datetime | None = None
    if dt_str is not None:
        result = parser.parse(dt_str, **kwargs)

    return result



from types import TracebackType
from typing import Type
import os
import traceback


def exc_format(exc: Exception, exc_info: tuple[Type[BaseException], BaseException, TracebackType]) -> str:
    """
    Formata a mensagem de erro decorrente da exceção levantada em tempo de execução, no formato:

    <python_module>, <line_number>: <exc_class> - <exc_text>

    :param exc: a exceção levantada
    :param exc_info: informações associadas à exceção
    :return: a mensagem de erro formatada
    """
    tback: TracebackType = exc_info[2]
    cls: str = str(exc.__class__)

    # obtem o ponto de execução que provocou a exceção (última posição na pilha)
    tlast: traceback = tback
    while tlast.tb_next is not None:
        tlast = tlast.tb_next

    # obtem nome do módulo e linha do código (origem da exceção)
    try:
        fname: str = os.path.split(tlast.tb_frame.f_code.co_filename)[1]
    except Exception:
        fname: str = "<unknow module>"
    fline: int = tlast.tb_lineno
    return f"{fname}, {fline}, {cls[8:-2]} - {exc}"



from email.message import EmailMessage
from smtplib import SMTP
from typing import Final
import sys
from .env_pomes import APP_PREFIX, env_get_str, env_get_int

EMAIL_ACCOUNT: Final[str] = env_get_str(f"{APP_PREFIX}_EMAIL_ACCOUNT")
EMAIL_PWD: Final[str] = env_get_str(f"{APP_PREFIX}_EMAIL_PWD")
EMAIL_PORT: Final[int] = env_get_int(f"{APP_PREFIX}_EMAIL_PORT")
EMAIL_SERVER: Final[str] = env_get_str(f"{APP_PREFIX}_EMAIL_SERVER")


def email_send(errors: list[str], user_email: str, subject: str, content: str):

    # import needed function
    from .exception_pomes import exc_format

    # connstrói o email
    email_msg = EmailMessage()
    email_msg.set_content(content)
    email_msg["Subject"] = subject
    email_msg["From"] = EMAIL_ACCOUNT
    email_msg["To"] = user_email

    # instancia o email server
    server = SMTP(host=EMAIL_SERVER, port=EMAIL_PORT)
    server.starttls()
    server.login(user=EMAIL_ACCOUNT, password=EMAIL_PWD)

    # envia a mensagem
    try:
        server.send_message(email_msg)
        server.quit()
    except Exception as e:
        # a operação resultou em exceção
        errors.append(f"Erro no envio de email: {exc_format(e, sys.exc_info())}")



from typing import Iterable
import base64


def jsonify_dict(source: dict):
    """
    Transforma os valores no dict *source* em valores que possam ser serializados em JSON, evitando *TypeError*:

    - *bytes* e *bytearray* são transformados em *str* formato *Base64*
    - *Iterable* é transformado em *list*
    - os outros tipos são mantidos
    HAZARD: dependendo do tipo de objeto contido em *source*, o resultado final poderá não ser serializável.

    :param source: o dict a ser tornado serializável
    """
    for key, value in source.items():
        if isinstance(value, dict):
            jsonify_dict(value)
        elif isinstance(value, bytes) or isinstance(value, bytearray):
            source[key] = base64.b64encode(value).decode()
        elif isinstance(value, Iterable) and not isinstance(value, str):
            source[key] = jsonify_iterable(value)


def jsonify_iterable(source: Iterable) -> list[any]:
    """
    Transforma os valores na lista *source* em valores que possam ser serializados em JSON, evitando *TypeError*:

    - *bytes* e *bytearray* são transformados em *str* formato *Base64*
    - *Iterable* é transformado em *list*
    - os outros tipos são mantidos
    HAZARD: dependendo do tipo de objeto contido em *source*, o resultado final poderá não ser serializável.

    :param source: a lista a ser tornada serializável
    """
    result: list[any] = []
    for value in source:
        if isinstance(value, dict):
            jsonify_dict(value)
            result.append(value)
        elif isinstance(value, bytes) or isinstance(value, bytearray):
            result.append(base64.b64encode(value).decode())
        elif isinstance(value, Iterable) and not isinstance(value, str):
            result.append(jsonify_iterable(value))
        else:
            result.append(value)

    return result



from .datetime_pomes import (
    DATE_FORMAT_STD, DATE_FORMAT_COMPACT, DATE_FORMAT_INV,
    DATETIME_FORMAT_STD, DATETIME_FORMAT_COMPACT, DATETIME_FORMAT_INV,
    date_reformat, date_parse, datetime_parse
)
from .dict_pomes import (
    dict_has_key_chain, dict_get_value, dict_set_value, dict_reduce,
    dict_listify, dict_transform, dict_merge, dict_coalesce, dict_get_key,
    dict_get_keys, dict_from_object, dict_from_form, dict_from_list, dict_replace_value, dict_pop_value
)
from .email_pomes import (
    EMAIL_ACCOUNT, EMAIL_PWD, EMAIL_PORT, EMAIL_SERVER, email_send
)
from .encoding_pomes import (
    encode_ascii_hex, decode_ascii_hex
)
from .env_pomes import (
    APP_PREFIX, env_get_str, env_get_int, env_get_bool, env_get_float
)
from .exception_pomes import (
    exc_format
)
from .file_pomes import (
    file_from_request
)
from .http_pomes import (
    MIMETYPE_BINARY, MIMETYPE_CSS, MIMETYPE_CSV, MIMETYPE_HTML, MIMETYPE_JAVASCRIPT,
    MIMETYPE_JSON, MIMETYPE_MULTIPART, MIMETYPE_PDF, MIMETYPE_PKCS7, MIMETYPE_SOAP,
    MIMETYPE_TEXT, MIMETYPE_URLENCODED, MIMETYPE_XML, MIMETYPE_ZIP,
    http_status
)
from .json_pomes import (
    jsonify_dict, jsonify_iterable
)
from .list_pomes import (
    list_compare, list_flatten, list_unflatten,
    list_find_coupled, list_elem_starting_with, list_transform
)
from .str_pomes import (
    str_between, str_split_on_mark, str_find_whitespace
)
from .validation_pomes import (
    VALIDATION_MSG_LANGUAGE, VALIDATION_MSG_PREFIX,
    validate_value, validate_bool, validate_int, validate_float, validate_str,
    validate_date, validate_datetime, validate_ints, validate_strs,
    validate_format_error, validate_format_errors
)
from .xml_pomes import (
    XML_FILE_HEADER, xml_to_dict, xml_normalize_keys
)

__all__ = [
    # datetime_pomes
    DATE_FORMAT_STD, DATE_FORMAT_COMPACT, DATE_FORMAT_INV,
    DATETIME_FORMAT_STD, DATETIME_FORMAT_COMPACT, DATETIME_FORMAT_INV,
    date_reformat, date_parse, datetime_parse,
    # dict_pomes
    dict_has_key_chain, dict_get_value, dict_set_value, dict_reduce,
    dict_listify, dict_transform, dict_merge, dict_coalesce, dict_get_key,
    dict_get_keys, dict_from_object, dict_from_form, dict_from_list, dict_replace_value, dict_pop_value,
    # email_pomes
    EMAIL_ACCOUNT, EMAIL_PWD, EMAIL_PORT, EMAIL_SERVER, email_send,
    # encoding_pomes
    encode_ascii_hex, decode_ascii_hex,
    # env_pomes
    APP_PREFIX, env_get_str, env_get_int, env_get_bool, env_get_float,
    # exception_pomes
    exc_format,
    # file_pomes
    file_from_request,
    # http_pomes
    MIMETYPE_BINARY, MIMETYPE_CSS, MIMETYPE_CSV, MIMETYPE_HTML, MIMETYPE_JAVASCRIPT,
    MIMETYPE_JSON, MIMETYPE_MULTIPART, MIMETYPE_PDF, MIMETYPE_PKCS7, MIMETYPE_SOAP,
    MIMETYPE_TEXT, MIMETYPE_URLENCODED, MIMETYPE_XML, MIMETYPE_ZIP,
    http_status,
    # json_pomes
    jsonify_dict, jsonify_iterable,
    # list_pomes
    list_compare, list_flatten, list_unflatten,
    list_find_coupled, list_elem_starting_with, list_transform,
    # validation_pomes
    VALIDATION_MSG_LANGUAGE, VALIDATION_MSG_PREFIX,
    validate_value, validate_bool, validate_int, validate_float, validate_str,
    validate_date, validate_datetime, validate_ints, validate_strs,
    validate_format_error, validate_format_errors,
    # str_pomes
    str_between, str_split_on_mark, str_find_whitespace,
    # xml_pomes
    XML_FILE_HEADER, xml_to_dict, xml_normalize_keys
]

__version__ = "0.1.4"
__version_info__ = (0, 1, 4)



from flask import Request
from werkzeug.datastructures import FileStorage


def file_from_request(request: Request, file_name: str = None, file_seq: int = 0) -> bytes:

    # inicializa a variável de retorno
    result: bytes | None = None

    count: int = len(request.files)
    # algum arquivo foi encontrado ?
    if count > 0:
        # sim, obtenha o arquivo
        file: FileStorage | None = None
        if isinstance(file_name, str):
            file = request.files.get(file_name)
        elif isinstance(file_seq, int) and file_seq >= 0:
            file_in: str = list(request.files)[file_seq]
            file = request.files[file_in]

        if file is not None:
            result: bytes = file.stream.read()

    return result
